/**
 *******************************************************************************
 * @file    tmpm341_phc.c
 * @brief   This file provides API functions for PHC driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm341_phc.h"

#if defined (__TMPM341_PHC_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup PHC 
  * @brief PHC driver modules
  * @{
  */
/** @defgroup PHC_Private_Defines
  * @{
  */

#define EN_PHCEN_SET        ((uint32_t)0x00000001)
#define EN_PHCEN_CLEAR      ((uint32_t)0xFFFFFFFE)

#define CR_MODE_CLEAR       ((uint32_t)0xFFFFFFFC)
#define CR_NF_CLEAR         ((uint32_t)0xFFFFFFFB)
#define FLG_CMP0_MASK       ((uint32_t)0x00000001)
#define FLG_CMP1_MASK       ((uint32_t)0x00000002)
#define FLG_OVF_MASK        ((uint32_t)0x00000004)
#define FLG_UDF_MASK        ((uint32_t)0x00000008)

#define RUN_CNT_CLEAR       ((uint32_t)0xFFFFFFFD)
#define PHC_FLG_MASK        ((uint32_t)0x0000000F)

/** @} */
/* End of group PHC_Private_Defines */

/** @defgroup PHC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group PHC_Private_FunctionPrototypes */

/** @defgroup PHC_Private_Functions
  * @{
  */

/** @} */
/* End of group group PHC_Private_Functions */

/** @defgroup PHC_Exported_Functions
  * @{
  */

/**
  * @brief  Enable the specified PHC channel.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @retval None
  */
void PHC_Enable(TSB_PHC_TypeDef * PHCx)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));

    /* Set PHCxEN<PHCEN> to enable PHCx */
    PHCx->EN |= EN_PHCEN_SET;
}

/**
  * @brief  Disable the specified PHC channel.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @retval None
  */
void PHC_Disable(TSB_PHC_TypeDef * PHCx)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));

    /* Clear PHCxEN<PHCEN> to disable PHCx */
    PHCx->EN &= EN_PHCEN_CLEAR;
}

/**
  * @brief  Start or stop counter of the specified PHC channel.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  Cmd: The command for the counter.
  *   This parameter can be PHC_RUN or PHC_STOP.
  * @retval None
  */
void PHC_SetRunState(TSB_PHC_TypeDef * PHCx, uint32_t Cmd)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_CMD(Cmd));

    /* Write command into RUN register */
    PHCx->RUN = Cmd;
}

/**
  * @brief  Initialize the specified PHC channel.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  InitStruct: The structure containing basic PHC configuration.
  * @retval None
  */
void PHC_Init(TSB_PHC_TypeDef * PHCx, PHC_InitTypeDef * InitStruct)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_CR_MODE(InitStruct->Mode));
    assert_param(IS_PHC_CR_NOISEFILTER(InitStruct->NoiseFilterCtrl));
    assert_param(IS_PHC_COUNT_CTRL(InitStruct->CountClearCtrl));

    if ((PHCx->RUN & PHC_RUN) == 0U) {  /* stopping? */
        /* Configure mode for PHCx */
        tmp = PHCx->CR;
        tmp &= CR_MODE_CLEAR;
        tmp &= CR_NF_CLEAR;
        tmp |= InitStruct->Mode;
        tmp |= InitStruct->NoiseFilterCtrl;
        PHCx->CR = tmp;

        /* Control counter clear for PHCx */
        tmp = PHCx->RUN;
        tmp &= RUN_CNT_CLEAR;
        tmp |= InitStruct->CountClearCtrl;
        PHCx->RUN = tmp;
    }
}

/**
  * @brief  Indicate what causes the interrupt.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @retval The interrupt factor of PHCx.
  */
PHC_INTFactor PHC_GetINTFactor(TSB_PHC_TypeDef * PHCx)
{
    PHC_INTFactor retval = { 0U };

    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));

    /* Get interrupt factor */
    retval.All = PHCx->FLG & PHC_FLG_MASK;
    return retval;
}

/**
  * @brief  Clear specified interrupt factor flag
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  ClearINT: Select the clear of PHC interrupt.
  *   This parameter can be:
  *   PHC_FLG_CMP0, PHC_FLG_CMP1,PHC_FLG_OVERFLOW, PHC_FLG_UNDERFLOW
  * @retval None
  */
void PHC_ClearINTFactor(TSB_PHC_TypeDef * PHCx, uint32_t ClearINT)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_FLG(ClearINT));

    /* Clear the specified interrupt */
    PHCx->FLG |= ClearINT;
}

/**
  * @brief  Enable some PHC interrupt.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  EnableINT: Select the enable of PHC interrupt.
  *   This parameter can be one of the following values:
  *   PHC_CR_INT_COMP0, PHC_CR_INT_COMP1,
  *   PHC_CR_INT_COMP0_AND_1, PHC_CR_INT_EVERY
  *   or PHC_CR_INT_ALL.
  * @retval None
  */
void PHC_EnableInterrupt(TSB_PHC_TypeDef * PHCx, uint32_t EnableINT)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_INT_ENABLE(EnableINT));

    /* Enable the specified interrupt */
    PHCx->CR |= EnableINT;
}

/**
  * @brief  Disable some PHC interrupt.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  DisableINT: Select the disable of PHC interrupt.
  *   This parameter can be one of the following values:
  *   PHC_CR_INT_COMP0, PHC_CR_INT_COMP1,
  *   PHC_CR_INT_COMP0_AND_1, PHC_CR_INT_EVERY
  *   or PHC_CR_INT_ALL.
  * @retval None
  */
void PHC_DisableInterrupt(TSB_PHC_TypeDef * PHCx, uint32_t DisableINT)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_INT_ENABLE(DisableINT));

    /* Mask the specified interrupt */
    PHCx->CR &= ~DisableINT;
}

/**
  * @brief  Get Pulse-counter value of PHCx.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @retval Pulse-counter value of PHCx
  */
uint16_t PHC_GetPulseCntValue(TSB_PHC_TypeDef * PHCx)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));

    /* Return pulse-counter value */
    return (uint16_t) PHCx->CNT;
}

/**
  * @brief  Clear Pulse-counter value of PHCx.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @retval None.
  */
void PHC_ClearPulseCntValue(TSB_PHC_TypeDef * PHCx)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));

    if ((PHCx->RUN & PHC_RUN) == 0U) {  /* stopping? */
        /* Clear pulse-counter for PHCx */
        PHCx->RUN |= PHC_COUNT_CLR;
    }
}

/**
  * @brief  Get PHC compare value of PHCx.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  CmpReg: Select the compare register to read.
  *   This parameter can be: PHC_COMP_0 or PHC_COMP_1.
  * @retval Capture value of PHCx
  */
uint16_t PHC_GetCompareValue(TSB_PHC_TypeDef * PHCx, uint8_t CmpReg)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_COMP_REG(CmpReg));

    /* Return compare value */
    return CmpReg ? (uint16_t) PHCx->CMP1 : (uint16_t) PHCx->CMP0;
}

/**
  * @brief  Set PHC compare value of PHCx.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3.
  * @param  CmpReg: Select the compare register to read.
  *   This parameter can be: PHC_COMP_0 or PHC_COMP_1.
  * @param  CmpValue: the compare register value.
  *   This parameter can be: 0x0000 to 0xFFFF.
  * @retval None.
  */
void PHC_SetCompareValue(TSB_PHC_TypeDef * PHCx, uint8_t CmpReg, uint16_t CmpValue)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_PHC_COMP_REG(CmpReg));
    /* assert_param(IS_PHC_CMP_VALUE(CmpValue)); *//* uint16_t */

    /* Set compare register */
    if (CmpReg == PHC_COMP_0) {
        PHCx->CMP0 = (uint32_t) CmpValue;
    } else {
        PHCx->CMP1 = (uint32_t) CmpValue;
    }
}

/**
  * @brief  Enable or disable DMA request.
  * @param  PHCx: Select the PHC channel.
  *   This parameter can be one of the following values:
  *   TSB_PHC0, TSB_PHC1, TSB_PHC2, TSB_PHC3
  * @param  NewState: New state of DMA request.
  *   This parameter can be ENABLE or DISABLE.
  * @param  DMAReq: DMA request.
  *   This parameter can be PHC_DMA_REQ_CAPTURE_2.
  * @retval None
  */
void PHC_SetDMAReq(TSB_PHC_TypeDef * PHCx, FunctionalState NewState, uint8_t DMAReq)
{
    /* Check the parameters */
    assert_param(IS_PHC_ALL_PERIPH(PHCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_PHC_DMA_REQ(DMAReq));

    if (NewState == ENABLE) {
        PHCx->DMA |= (uint32_t) DMAReq;
    } else {
        PHCx->DMA &= ~((uint32_t) DMAReq);
    }
}

/** @} */
/* End of group PHC_Exported_Functions */

/** @} */
/* End of group PHC */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM341_PHC_H) */
